@@ -618,8 +618,9 @@ lysc_ident_derived_unlink(const struct lysc_ident *ident)
     const struct lysp_submodule *submod;
     const struct lysp_module *base_pmod = NULL;
     const struct lysp_ident *identp = NULL;
-    const struct lys_module *mod;
+    const struct lys_module *mod, *iter;
     const char *base_name;
+    uint32_t i;
 
     /* find the parsed identity */
     LY_ARRAY_FOR(ident->module->parsed->identities, u) {
@@ -658,6 +659,17 @@ lysc_ident_derived_unlink(const struct lysc_ident *ident)
         }
         ++base_name;
 
+        i = 0;
+        while ((iter = ly_ctx_get_module_iter(ident->module->ctx, &i))) {
+            if (iter == mod) {
+                break;
+            }
+        }
+        if (!iter) {
+            /* target module was freed already */
+            continue;
+        }
+
         /* find the compiled base */
         LY_ARRAY_FOR(mod->identities, v) {
             if (!strcmp(mod->identities[v].name, base_name)) {
@@ -666,7 +678,10 @@ lysc_ident_derived_unlink(const struct lysc_ident *ident)
                     if (mod->identities[v].derived[w] == ident) {
                         /* remove the link */
                         LY_ARRAY_DECREMENT(mod->identities[v].derived);
-                        if (w < LY_ARRAY_COUNT(mod->identities[v].derived)) {
+                        if (!LY_ARRAY_COUNT(mod->identities[v].derived)) {
+                            LY_ARRAY_FREE(mod->identities[v].derived);
+                            mod->identities[v].derived = NULL;
+                        } else if (w < LY_ARRAY_COUNT(mod->identities[v].derived)) {
                             memmove(mod->identities[v].derived + w, mod->identities[v].derived + w + 1,
                                     (LY_ARRAY_COUNT(mod->identities[v].derived) - w) * sizeof ident);
                         }
@@ -700,7 +715,7 @@ lysc_range_free(struct ly_ctx *ctx, struct lysc_range *range)
     FREE_ARRAY(ctx, range->exts, lysc_ext_instance_free);
 }
 
-static void
+void
 lysc_pattern_free(struct ly_ctx *ctx, struct lysc_pattern **pattern)
 {
     if (--(*pattern)->refcount) {
